import color from 'color';
import _extends from '@babel/runtime/helpers/extends';
import _inheritsLoose from '@babel/runtime/helpers/inheritsLoose';
import _assertThisInitialized from '@babel/runtime/helpers/assertThisInitialized';
import React from 'react';
import PropTypes from 'prop-types';
import { AppContainerUtils, AppContainer } from 'fractal-component';
import jss from 'jss';
import jssDefaultPreset from 'jss-preset-default';

var INCREASE_COUNT =
/*#__PURE__*/
Symbol("INCREASE_COUNT");

var actionTypes = /*#__PURE__*/Object.freeze({
    INCREASE_COUNT: INCREASE_COUNT
});

function increaseCount() {
  return {
    type: INCREASE_COUNT
  };
}

var index = /*#__PURE__*/Object.freeze({
    increaseCount: increaseCount
});

var styles = {
  table: {
    display: "flex",
    "flex-wrap": "wrap",
    margin: "0.2em 0.2em 0.2em 0.2em",
    padding: 0,
    "flex-direction": "column",
    width: "20em"
  },
  cell: {
    "box-sizing": "border-box",
    "flex-grow": 1,
    width: "100%",
    overflow: "hidden",
    padding: "0.2em 0.2em",
    border: "solid 2px " +
    /*#__PURE__*/
    color("slategrey").fade(0.5),
    "border-bottom": "none",
    "background-color": "#f7f7f7",
    display: "flex",
    "align-items": "center",
    "justify-content": "center",
    "&:last-child": {
      "border-bottom": "solid 2px " +
      /*#__PURE__*/
      color("slategrey").fade(0.5) + " !important"
    },
    "&$counter-container": {
      height: "16.5em",
      "& span": {
        "font-size": "40px"
      }
    }
  },
  "counter-container": {}
};

var Counter =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Counter, _React$Component);

  function Counter(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      count: 0
    };
    _this.styleSheet = null;
    _this.componentManager = AppContainerUtils.registerComponent(_assertThisInitialized(_assertThisInitialized(_this)), {
      namespace: "io.github.t83714/Counter",
      reducer: function reducer(state, action) {
        switch (action.type) {
          case INCREASE_COUNT:
            {
              var toggleButtonActive = action.toggleButtonActive;
              var incresement = state.count >= 10 && toggleButtonActive ? 2 : 1;
              return _extends({}, state, {
                count: state.count + incresement
              });
            }

          default:
            return state;
        }
      },

      /**
       * Register actions for action serialisation / deserialisation.
       * It's much easier to use Symbol as action type to make sure no action type collision among different component.
       * ( Considering we now use actions as primary way for inter-component communication, it's quite important in a multicaset action environment)
       */
      actionTypes: actionTypes,
      // --- specify accepted types of external multicast actions
      // --- By default, component will not accept any incoming multicast action.
      // --- No limit to actions that are sent out
      allowedIncomingMulticastActionTypes: [INCREASE_COUNT],
      // --- `namespaceInitCallback` is guaranteed only to be called once
      namespaceInitCallback: function namespaceInitCallback(componentManager) {
        var jssRef;

        if (!props.styles) {
          // --- if use built-in style, we want to make sure this component use its own jss setting
          jssRef = jss.setup(jssDefaultPreset());
        } else {
          jssRef = jss;
        }

        var styleSheet = jssRef.createStyleSheet(props.styles ? props.styles : styles, {
          generateClassName: componentManager.createClassNameGenerator()
        }).attach();
        return {
          styleSheet: styleSheet
        };
      },
      namespaceDestroyCallback: function namespaceDestroyCallback(_ref) {
        var styleSheet = _ref.styleSheet;
        styleSheet.detach();
      }
    });
    return _this;
  }

  var _proto = Counter.prototype;

  _proto.render = function render() {
    var _this$componentManage = this.componentManager.getNamespaceData(),
        styleSheet = _this$componentManage.styleSheet;

    var classes = styleSheet.classes;
    return React.createElement("div", {
      className: classes.table
    }, React.createElement("div", {
      className: classes.cell
    }, "Counter"), React.createElement("div", {
      className: classes.cell + " " + classes["counter-container"]
    }, React.createElement("span", null, this.state.count)));
  };

  return Counter;
}(React.Component);

Counter.propTypes = {
  styles: PropTypes.object,
  appContainer:
  /*#__PURE__*/
  PropTypes.instanceOf(AppContainer)
};

export default Counter;
export { actionTypes, index as actions };
